from abc import ABC, abstractmethod
import threading

from authlib.oauth2.rfc6749 import OAuth2Token
from authlib.integrations.requests_client import OAuth2Session

from .constants import OPENID_SCOPE
from .discovery import ArthurOIDCMetadata


class ArthurAPISession(ABC):
    """
    The ArthurSession is an interface that allows users to retrieve a token for use with the Arthur API.
    Calling the token() method may refresh the token if it's within leeway seconds of expiration.
    The session object is thread safe, only one thread will be able to refresh it at a time.
    """

    @abstractmethod
    def token(self, leeway=30) -> OAuth2Token:
        pass

    @abstractmethod
    def host(self) -> str:
        pass


class _ArthurRefreshTokenAPISession(ArthurAPISession):
    """
    This class takes an OAuth2Session and creates a wrapper to automatically refresh it when asked for.
    Note, this is not meant to be used directly, but should be returned from other authorization flows.
    """

    def __init__(
        self,
        oauth_session: OAuth2Session,
        metadata: ArthurOIDCMetadata,
    ):
        self._oauth_session = oauth_session
        self._metadata = metadata
        self._lock = threading.Lock()

    def token(self, leeway=30) -> OAuth2Token:
        with self._lock:
            if self._oauth_session.token.is_expired(leeway):
                self._oauth_session.refresh_token(url=self._metadata.token_endpoint)
            return self._oauth_session.token

    def host(self) -> str:
        return self._metadata.arthur_host


class ArthurClientCredentialsAPISession(ArthurAPISession):
    """
    This class takes an OAuth2 client ID and secret and creates a wrapper to automatically refresh the token
    when asked for.
    """

    def __init__(
        self,
        client_id: str,
        client_secret: str,
        metadata: ArthurOIDCMetadata,
        verify: bool = True,
    ):
        self.client_id = client_id
        self.client_secret = client_secret
        self._oauth_session = OAuth2Session(
            client_id=self.client_id,
            client_secret=self.client_secret,
            scope=OPENID_SCOPE,
        )
        self._oauth_session.session.verify = verify
        self._metadata = metadata
        self._lock = threading.Lock()

    def token(self, leeway=30) -> OAuth2Token:
        with self._lock:
            if (
                self._oauth_session.token is None
                or self._oauth_session.token.is_expired(leeway)
            ):
                self._oauth_session.fetch_token(url=self._metadata.token_endpoint)
            return self._oauth_session.token

    def host(self) -> str:
        return self._metadata.arthur_host



# coding: utf-8

"""
    Arthur Scope

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from arthur_client.api_bindings.models.post_data_retrieval_operation import PostDataRetrievalOperation

class TestPostDataRetrievalOperation(unittest.TestCase):
    """PostDataRetrievalOperation unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> PostDataRetrievalOperation:
        """Test PostDataRetrievalOperation
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `PostDataRetrievalOperation`
        """
        model = PostDataRetrievalOperation()
        if include_optional:
            return PostDataRetrievalOperation(
                start_timestamp = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'),
                end_timestamp = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'),
                data_retrieval_filters = [
                    arthur_client.api_bindings.models.data_result_filter.DataResultFilter(
                        field_name = '', 
                        op = 'greater_than', 
                        value = null, )
                    ],
                pagination_options = arthur_client.api_bindings.models.connector_pagination_options.ConnectorPaginationOptions(
                    page = 1.0, 
                    page_size = 1.0, )
            )
        else:
            return PostDataRetrievalOperation(
                start_timestamp = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'),
                end_timestamp = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'),
        )
        """

    def testPostDataRetrievalOperation(self):
        """Test PostDataRetrievalOperation"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Arthur Scope

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 0.1.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self


class ScopeSchemaTag(str, Enum):
    """
    ScopeSchemaTag
    """

    """
    allowed enum values
    """
    LLM_CONTEXT = 'llm_context'
    LLM_PROMPT = 'llm_prompt'
    LLM_RESPONSE = 'llm_response'
    PRIMARY_TIMESTAMP = 'primary_timestamp'
    CATEGORICAL = 'categorical'
    CONTINUOUS = 'continuous'
    PREDICTION = 'prediction'
    GROUND_TRUTH = 'ground_truth'
    PIN_IN_DEEP_DIVE = 'pin_in_deep_dive'
    POSSIBLE_SEGMENTATION = 'possible_segmentation'

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of ScopeSchemaTag from a JSON string"""
        return cls(json.loads(json_str))


